/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef MESSAGEDISPATCHER_H
#define MESSAGEDISPATCHER_H

#include <qobject.h>
#include <qobjectlist.h>
#include <qdict.h>
#include <qstring.h>
#include <qguardedptr.h>

#include "message.h"

namespace IRC
{

class MessageDispatcher : public QObject
{
    Q_OBJECT
public:
    enum MessageReceiverType { PermanentMessageReceiver, TemporaryMessageReceiver };

    MessageDispatcher( QObject *parent = 0, const char *name = 0 );

    void registerMessageReceiver( QObject *receiver, MessageReceiverType receiverType = PermanentMessageReceiver );
    void registerMessageReceiver( const QString &key, QObject *receiver, MessageReceiverType receiverType = PermanentMessageReceiver );

    static QStringList handledMessageTypesForReceiver( QObject *receiver );

public slots:
    void unregisterMessageReceiver( QObject *receiver );

    void dispatchMessage( const IRC::Message &msg );
    void dispatchMessage( const QString &key, const IRC::Message &msg );

signals:
    void unhandledMessage( const IRC::Message &msg );

private slots:
    void receiverDestroyed( QObject *receiver );

private:
    bool unregisterMessageReceiverDirect( QObject *receiver );

    static QStringList messageTypesFromSlots( const QStrList &slotNames );
    static QString messageTypeFromSlot( const QString &slot );
    QString slotForMessage( const Message &msg );

    QDict<QObjectList> m_typeToReceiver;
    QPtrList<QObject> m_receivers;

    QGuardedPtr<QObject> m_temporaryReceiver;
    QStringList m_keysOfTemporaryReceiver;
};

}

#endif // MESSAGEDISPATCHER_H

/* vim: et sw=4 ts=4
 */
